### this is the default makefile for DOS: default.mk

SUFFIXES = .cpp .c .asm .l .y .ec
.SUFFIXES: $(SUFFIXES)

#	ASM section
AS	 = masm
ASFLAGS	 = /mx			# don't convert to upper case
COMPILE.s= $(AS) $(ASFLAGS)
.asm .asm.com:
	 $(COMPILE.s) $<;
	 $(LINK.c) -o $*.exe $*.obj $(LDFLAGS)
	 $(RM) $*.obj
	 exe2bin $*.exe $*.com
	 $(RM) $*.exe
asm.exe:
	 $(COMPILE.s) $<;
	 $(LINK.c) -o $@ $*.obj $(LDFLAGS)
	 $(RM) $*.obj
.asm.obj:
	 $(COMPILE.s) $<;
.asm.o:
	 $(COMPILE.s) $<;
	 mv $*.obj $@

#	C section
CC	 = cl -A$(MODEL)
CFLAGS	 = /Ox /G2		# full optimization, 80286 opcodes
CDEBUG	 = /Od /Zi		# no optimization, Codeview debuggable
COMPILE.c= $(CC) $(CFLAGS) -c
.c.com:
	 $(COMPILE.cpp) -mt $< $(LIBS)
	 $(RM) $*.obj
.c .c.exe:
	 $(LINK.c) -o $@ $< $(LDFLAGS)
	 $(RM) $*.obj
.c.o:
	 $(COMPILE.c) /Fo$*.o $<
.c.obj:
	 $(COMPILE.c) $<

#	D section (ANSI C declaration headers)
.c.d:
	 echo "#ifndef NOPROTOTYPES" > $*.tmp
	-$(COMPILE.c) -DNOPROTOTYPES -Zg $< >> $*.tmp
	 echo "#endif" >> $*.tmp
	+updexhdr $*.tmp $*.d
	 $(RM) $*.tmp

#	I section (preprocessed C files)
.c.i:
	 $(COMPILE.c) -P $<

#	CPP Section
CPPINCDIR= c:\zortech\include
CPP	 = ztc -DNO_EXT_KEYS -I$(CPPINCDIR) -B -m$(MODEL)i	# integer only
CPPF	 = ztc -DNO_EXT_KEYS -I$(CPPINCDIR) -B -m$(MODEL)	# floats also
CPPFLAGS = -o					# full optimization
CPPDEBUG = -g -co -s				# codeview and stack checking
COMPILE.cpp=$(CPP) $(CPPFLAGS)
.cpp .cpp.com:
	 $(COMPILE.cpp) -mt $< $(LIBS)
	 $(RM) $*.obj
.cpp.exe:
	 $(COMPILE.cpp) $<
	 $(RM) $*.obj
.cpp.obj:
	 $(COMPILE.cpp) -c $<
.cpp.o:
	 $(COMPILE.cpp) -c $<
	 $(MV) $*.obj $@
.cpp.c:
	$(COMPILE.cpp) -c -e -l$*.c $<

#	ESQL/C section
ESQL	 = esql
EFLAGS	 =
COMPILE.e= $(ESQL) $(EFLAGS)
.ec.com:
	 $(COMPILE.e) -c $<
	 $(COMPILE.cpp) -mt $*.c $(LIBS) $(MODEL)libsql.lib
	 $(RM) $*.obj $*.c
.ec .ec.exe:
	 $(COMPILE.e) -e $<
	 $(LINK.c) -o $@ $*.c $(LDFLAGS) $(MODEL)libsql
	 $(RM) $*.obj $*.c
.ec.o:
	 $(COMPILE.e) -e $<
	 $(COMPILE.c) /Fo$*.o $*.c
	 $(RM) $*.c
.ec.obj:
	 $(COMPILE.e) -e $<
	 $(COMPILE.c) $*.c
	 $(RM) $*.c
.ec.c:
	 $(COMPILE.e) -e $<

#	Lex section
LEX	 = flex
LFLAGS	 = -I				# interactive scanner
LEX.l	 = $(LEX) $(LFLAGS) -t
.l.com:
	 $(LEX.l) $< > $*.c
	 $(COMPILE.cpp) -mt $*.c $(LIBS)
	 $(RM) $*.obj $*.c
.l .l.exe:
	 $(LEX.l) $< > $*.c
	 $(LINK.c) -o $@ $*.c $(LDFLAGS)
	 $(RM) $*.obj $*.c
.l.o:
	 $(LEX.l) $< > $*.c
	 $(COMPILE.c) /Fo$*.o $*.c
	 $(RM) $*.c
.l.obj:
	 $(LEX.l) $< > $*.c
	 $(COMPILE.c) $*.c
	 $(RM) $*.c
.l.c:
	 $(LEX.l) $< > $@

#	YACC Section
YACC	 = yacc
YFLAGS	 =
YACC.y	 = $(YACC) $(YFLAGS)
.y.com:
	 $(YACC.y) $<
	 $(MV) $(YTAB).c $*.c
	 $(COMPILE.cpp) -mt $*.c $(LIBS)
	 $(RM) $*.obj $*.c
.y .y.exe:
	 $(YACC.y) $<
	 $(LINK.c) -o $@ $(YTAB).c $(LDFLAGS)
	 $(RM) $(YTAB).obj $(YTAB).c
.y.o:
	 $(YACC.y) $<
	 $(COMPILE.c) /Fo$*.o $(YTAB).c
	 $(RM) $(YTAB).c
.y.obj:
	 $(YACC.y) $<
	 $(COMPILE.c) /Fo$*.obj $(YTAB).c
	 $(RM) $(YTAB).c
.y.h:
	 $(YACC.y) -d $<
	 $(RM) $(YTAB).c
	 $(MV) $(YTAB).h $@
.y.c:
	 $(YACC.y) $<
	 $(MV) $(YTAB).c $@

#	BSD Unix Misc section
#A        = .a
#AR       = ar
#ARFLAGS  =
#BIN      = /usr/local/bin
#CP       = cp
#E        =
#EDITOR   = /usr/bin/vi
#GFLAGS   =
#GET      = get
#LDEBUG   =
#LDFLAGS  =
#LD       = ld
#LIBDIR   =
#LINK.c   = $(CC) $(CFLAGS)
#MAKE     = make
#MKDEPEND = makedepend
#MODEL    =
#MV       = mv
#O        = .o
#RANLIB   = ranlib
#RM       = rm -f
#SHELL    = /bin/csh
#SHELLCMD =
#STACK    =
#YTAB     = y.tab

#	DOS Misc section
A        = .lib
AR       = lib
ARFLAGS  =
BIN      = C:\bin
CP       = cp
E        = .exe
EDITOR   = $(BIN)\vi
GFLAGS   =
GET      = echo cannot get
LDEBUG   = /link /noe /noi /co /st:$(STACK) $(LIBS)
LDFLAGS  = /link /noe /noi /st:$(STACK) $(LIBS)
LD       = $(CC) $(CFLAGS)
LIBDIR   = c:\msc5.1\lib
LINK.c   = $(CC) $(CFLAGS)
MAKE     = make
MKDEPEND = mkdepend
MODEL    = S
MV       = mv
O        = .o
RANLIB   = echo cannot ranlib 
RM       = rm -f
SHELL    = command /c
SHELLCMD = break call cd chcp chdir cls copy ctty date del dir \
	   echo erase exit for goto if md mkdir path pause prompt \
	   rd rem ren rename rmdir set shift time type ver verify vol
STACK    = 2000
YTAB     = ytab
